use domain::dto::feedback::FeedbackAdd;
use domain::entities::feedback::Feedback;
use domain::vo::feedback::FeedbackPageable;
use library::context::Context;
use library::db;
use library::model::response::ResResult;

/// 获取反馈信息列表
pub async fn get_feedback_list_by_page(
    context: Context,
    page: i64,
    page_size: i64,
) -> ResResult<FeedbackPageable> {
    if !context.account.unwrap().role.is_admin() {
        tracing::error!("非管理员用户，无法获取反馈信息列表");
        return Ok(FeedbackPageable::empty(page, page_size));
    }
    let feedback_list = Feedback::search_feedback(page, page_size, db!()).await.ok();
    if feedback_list.is_none() {
        tracing::error!("反馈信息为空");
        return Ok(FeedbackPageable::empty(page, page_size));
    }
    let total = get_feedback_count().await;
    Ok(FeedbackPageable::new(
        feedback_list.unwrap(),
        total,
        page,
        page_size,
    ))
}

/// 获取反馈信息总数
async fn get_feedback_count() -> i64 {
    let count = Feedback::count_feedback(db!()).await.ok();
    if count.is_none() {
        tracing::error!("反馈信息为空");
        return 0;
    }
    count.unwrap().count.unwrap()
}

/// 添加反馈信息
pub async fn add_feedback(context: Context, req: FeedbackAdd) -> ResResult<()> {
    let account = context.account.unwrap();
    let mut transaction = db!().begin().await?;
    match Feedback::add_feedback(
        &mut Feedback {
            user_id: account.id.clone(),
            content: req.content.unwrap(),
            ..Default::default()
        },
        &mut transaction,
    )
    .await
    {
        Ok(feedback) => {
            transaction.commit().await?;
            tracing::info!("添加反馈成功: {:?}", feedback)
        }
        Err(err) => {
            transaction.rollback().await?;
            tracing::error!(error = ?err, "添加反馈信息失败");
            return Err(library::model::response::ResErr::ErrService(None));
        }
    }

    Ok(())
}
